#include <stdio.h>
#include <assert.h>
#include <sys/time.h>

#include <fusb.h>

#include "USB_Device.hpp"

uint8_t bit_rev_8[256];

int main()
{
    for (int i = 0; i < 256; ++i)
    {
        bit_rev_8[i] = 0;
        for (int j = 0; j < 8; ++j)
        {
            if (i & (1 << j))
            {
                bit_rev_8[i] |= 1 << (7 - j);
            }
        }
    }
    
    USB_Device *dev = USB_Device::find_first(0x3141, 0x0007);
    if (!dev)
    {
        fprintf(stderr, "Device not found\n");
        return 1;
    }
    
    assert(dev->open());
    assert(dev->set_configuration(1));
    assert(dev->claim_interface(0));
    assert(dev->set_altinterface(1));
    
    fusb_devhandle *fdev = fusb_sysconfig::make_devhandle(dev->handle());
    assert(fdev);
    
    fusb_ephandle *ep = fdev->make_ephandle(6, true);
    assert(ep);
    
    ep->start();
    
    uint8_t buf[16384];
#if 0
    struct timeval start, end;
    gettimeofday(&start, 0);
    int bytes = 0;
    for (int i = 0; i < 100; ++i)
    {
        assert(ep->read(buf, sizeof(buf)) == sizeof(buf));
        bytes += sizeof(buf);
    }
    gettimeofday(&end, 0);
    int us = (end.tv_sec - start.tv_sec) * 1000000 + end.tv_usec - start.tv_usec;
    printf("%d bytes in %d us, %.3f MB/s\n", bytes, us, (float)bytes / us);
#endif
    
    uint8_t out[sizeof(buf)];
    while (true)
    {
        assert(ep->read(buf, sizeof(buf)) == sizeof(buf));
        for (unsigned int i = 0; i < sizeof(buf) / 2; ++i)
        {
            uint8_t pb = buf[i * 2];            // PB7:0 = {D7:13, 0}
            uint8_t pd = buf[i * 2 + 1];        // PD7:0 = {X, D0:6}
            int16_t sample = (bit_rev_8[pb] << 8) | bit_rev_8[pd];
            sample >>= 1;
            sample &= 0x3fff;
            sample -= 8192;
//             printf("%02x %02x %04x\n", pb, pd, sample);
            out[i * 2] = sample & 0xff;
            out[i * 2 + 1] = sample >> 8;
        }
       assert(write(1, out, sizeof(out)) == sizeof(out));
    }
    
    ep->stop();
    
    delete fdev;
    
    return 0;
}
